<?php

/*
 * plugin.nickname_sync.php
 * Description: This plugin synchronizes the nicknames of players in your localdatabase with
 *              nicknames from dedimania and gerymania records.
 * Version: 1.1
 * Date: 2011-03-27
 * Copyright (C) 2011 by .anDy
 * System: XASECO v1.12 and up
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program;
 * if not, see <http://www.gnu.org/licenses/>.
 *
 */


Aseco::registerEvent('onSync', 'ns_onSync');

// called @ onSync
function ns_onSync($aseco) {

    // check for required plugins
    $required = array('plugin.localdatabase.php' => true);
    foreach ($required as $plugin => &$state) {
        foreach ($aseco->plugins as $installed_plugin) {
            if ($plugin == $installed_plugin) {
                // Found, skip to next plugin
                continue 2;
            }
        }
        trigger_error('[plugin.nickname_sync.php] Unmet requirements! You need to activate "' . $plugin . '" in your "plugins.xml" to run this Plugin!', E_USER_ERROR);
    }
    // check for optional plugins
    $optional = array(
        'plugin.dedimania.php' => true,
        'plugin.gerymania.php' => true
    );
    $optionalCount = 0;
    foreach ($optional as $plugin => &$state) {
        foreach ($aseco->plugins as $installed_plugin) {
            if ($plugin == $installed_plugin) {
                // Found, skip to next plugin
                continue 2;
            }
        }
        $optionalCount++;
    }
    if (count($optional) == $optionalCount) {
        trigger_error('[plugin.nickname_sync.php] Plugin is currently useless! You need to activate "' . implode('" or "', array_keys($optional)) . '" in your "plugins.xml" to run this Plugin!', E_USER_WARNING);
    } else {
        // register events
        if (!in_array('ns_onNewChallenge2', $aseco->events['onNewChallenge2'])) {
            $aseco->registerEvent('onNewChallenge2', 'ns_onNewChallenge2');
        }
        if (!in_array('ns_onEverySecond', $aseco->events['onEverySecond'])) {
            $aseco->registerEvent('onEverySecond', 'ns_onEverySecond');
        }
    }

    // Register this to the global version pool (for up-to-date checks)
    $aseco->plugin_versions[] = array(
        'plugin' => 'plugin.nickname_sync.php',
        'author' => '.anDy',
        'version' => '1.1'
    );
}

// called @ onNewChallenge2
function ns_onNewChallenge2($aseco) {
    // register ns_onEverySecond-event for updating nicknames
    if (!in_array('ns_onEverySecond', $aseco->events['onEverySecond'])) {
        $aseco->registerEvent('onEverySecond', 'ns_onEverySecond');
    }
}

// called @ onEverySecond
// $dedi_db is imported from plugin.dedimania.php
// $gery_db is imported from plugin.gerymania.php
function ns_onEverySecond($aseco) {
    global $dedi_db, $gery_db;
    // on stuntmode compare with gerymania
    if ($aseco->server->gameinfo->mode == 4) {
        if (!empty($gery_db)) {
            ns_comparePlayers($gery_db, 'login', 'nick');
        }
    }
    // on other gamemodes compare with dedimania
    else {
        if (isset($dedi_db['Challenge']['Records'])) {
            ns_comparePlayers($dedi_db['Challenge']['Records'], 'Login', 'NickName');
        }
    }
}

function ns_comparePlayers($publics, $keyLogin, $keyNick) {
    global $aseco;
    $playersToUpdate = array();
    $playersUnknownPublic = array();
    foreach ($publics as $public) {
        // check if player is currently on server
        if (isset($aseco->server->players->player_list[$public[$keyLogin]])) {
            // player is online, no action
            continue;
        }
        // search nickname of player in local records
        foreach ($aseco->server->records->record_list as &$local) {
            if ($local->player->login == $public[$keyLogin]) {
                // check for different nickname
                if ($local->player->nickname != $public[$keyNick]) {
                    // update nickname in local records (impermanent, only for showing new nickname in widgets)
                    $local->player->nickname = $public[$keyNick];
                    // add it to the update list to make this nickname permament
                    $playersToUpdate[$public[$keyLogin]] = $public[$keyNick];
                }
                continue 2;
            }
        }
        // nickname not found, cache it for database search
        $playersUnknownPublic[$public[$keyLogin]] = $public[$keyNick];
        continue;
    }

    if (count($playersUnknownPublic) > 0) {
        // check if players exists on database
        $playersKnownLocal = ns_getPlayers($playersUnknownPublic);
        foreach ($playersKnownLocal as $login => $nick) {
            // check for different nickname
            if ($playersUnknownPublic[$login] != $nick) {
                // add it to upsdate list to make it permament
                $playersToUpdate[$login] = $playersUnknownPublic[$login];
            }
        }
        unset($playersKnownLocal);
    }
    unset($playersUnknownPublic);
    // update all new nicknames to databases
    ns_updatePlayers($playersToUpdate);
    // unregister ns_onEverySecond-event to speed up
    if (in_array('ns_onEverySecond', $aseco->events['onEverySecond'])) {
        unset($aseco->events['onEverySecond'][array_search('ns_onEverySecond', $aseco->events['onEverySecond'])]);
    }
}

function ns_updatePlayers($players) {
    global $aseco;
    if (count($players) > 0) {
        $msg = "[plugin.nickname_sync.php] Update nicknames of " . implode(", ", array_keys($players));
        $aseco->console($msg);

        foreach ($players as $login => $nick) {
            // update player data
            $query = 'UPDATE players
		          SET NickName=' . quotedString($nick) . '
                              WHERE Login=' . quotedString($login);
            $result = mysql_query($query);

            if (mysql_affected_rows() == -1) {
                trigger_error('[plugin.nickname_sync.php] Could not update player by public database! (' . mysql_error() . ')' . CRLF . 'sql = ' . $query, E_USER_WARNING);
            }
        }
    }
}

function ns_getPlayers($players) {
    $players = array();
    // find players from players-array in database
    $query = "SELECT Login, NickName FROM players WHERE Login IN ('" . implode("','", array_keys($players)) . "') ";
    $result = mysql_query($query);
    if ($result && mysql_num_rows($result) > 0) {
        while ($row = mysql_fetch_row($result)) {
            // row[0] = login, row[1] = nickname
            $players[$row[0]] = $row[1];
        }
    }
    // free up the results
    mysql_free_result($result);
    return $players;
}

?>